#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file Wheel_Control.py
 @brief Wheel_Control component
 @date $Date$


"""
import sys
import time
import serial
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist


# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
wheel_control_spec = ["implementation_id", "Wheel_Control", 
		 "type_name",         "Wheel_Control", 
		 "description",       "Wheel_Control component", 
		 "version",           "1.0.0", 
		 "vendor",            "VenderName", 
		 "category",          "Category", 
		 "activity_type",     "STATIC", 
		 "max_instance",      "1", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 ""]
# </rtc-template>

##
# @class Wheel_Control
# @brief Wheel_Control component
# 
# 
class Wheel_Control(OpenRTM_aist.DataFlowComponentBase):
	
	##
	# @brief constructor
	# @param manager Maneger Object
	# 
	def __init__(self, manager):
		OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

		VelValue_arg = [None] * ((len(RTC._d_TimedVelocity2D) - 4) / 2)
		self._d_VelValue = RTC.TimedVelocity2D(*VelValue_arg)
		"""
		"""
		self._VelIn = OpenRTM_aist.InPort("Vel", self._d_VelValue)


		


		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">
		
		# </rtc-template>


		 
	##
	#
	# The initialize action (on CREATED->ALIVE transition)
	# formaer rtc_init_entry() 
	# 
	# @return RTC::ReturnCode_t
	# 
	#
	def onInitialize(self):
		# Bind variables and configuration variable
		
		# Set InPort buffers
		self.addInPort("Vel",self._VelIn)
		
		# Set OutPort buffers
		
		# Set service provider to Ports
		
		# Set service consumers to Ports
		
		# Set CORBA Service Ports

		## gearbox
		self.radius = 0.046 #0.0275 #^C̔a
		self.tread =  0.14 #0.046   #^CԂ̋̔
		self.rpm = 8100 #6500      #[^̉]
		self.gear = 64 #114.7    #MA
		self.Maxrpm=int(3*(self.rpm/self.gear)) #3.0V̉]2
		self.Minrpm=0*(self.rpm/self.gear)      #0V̉]
		self.out_min = 1                        # dݒŏl
		self.out_max = 38                       # dݒől(3V)
		self.in_min = self.Minrpm               # ]ŏl
		self.in_max = self.Maxrpm               # ]ől
		self.right_sval=0
		self.left_sval =0
		
		return RTC.RTC_OK
	
	#	##
	#	# 
	#	# The finalize action (on ALIVE->END transition)
	#	# formaer rtc_exiting_entry()
	#	# 
	#	# @return RTC::ReturnCode_t
	#
	#	# 
	#def onFinalize(self):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The startup action when ExecutionContext startup
	#	# former rtc_starting_entry()
	#	# 
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onStartup(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The shutdown action when ExecutionContext stop
	#	# former rtc_stopping_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onShutdown(self, ec_id):
	#
	#	return RTC.RTC_OK
	
		##
		#
		# The activated action (Active state entry action)
		# former rtc_active_entry()
		#
		# @param ec_id target ExecutionContext Id
		# 
		# @return RTC::ReturnCode_t
		#
		#
	def onActivated(self, ec_id):
		print "onActivated"
		self.ser = serial.Serial('/dev/arduino',9600)
		return RTC.RTC_OK
	
		##
		#
		# The deactivated action (Active state exit action)
		# former rtc_active_exit()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onDeactivated(self, ec_id):
		print "onDeactivated"
		self.ser.close()
		return RTC.RTC_OK
	
		##
		#
		# The execution action that is invoked periodically
		# former rtc_active_do()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onExecute(self, ec_id):
		
		if self._VelIn.isNew():
			#lǂݍ
			readdata =self._VelIn.read()
			#O̒lۑ
			old_left_sval=self.left_sval
			old_right_sval=self.right_sval
			

			#E^ČvZ 
			#E[^̊px
			omega_r=(readdata.data.vx + self.tread * readdata.data.va)/self.radius
			print "omega_r"
			print omega_r
	
			#^ČvZ 
			#[^̊px
			omega_l=(readdata.data.vx - self.tread * readdata.data.va)/self.radius
			print "omega_l"
			print omega_l


			#[^̉]mF
			#(forward:f)
			if  omega_r>0 and omega_l>0: 
				self.ser.write("f") 
				#print "forward"

			#(turn right:r)
			elif omega_r<0 and omega_l>0: 
				self.ser.write("r")
				#print "right"

			#(turn left:l)
			elif omega_r>0 and omega_l<0: 
				self.ser.write("l")
				#print "left"

			#(back:b)
			elif omega_r<0 and omega_l<0:
				self.ser.write("b")
				#print "back"

			#(stop:s)
			else:
				self.ser.write("s")

			omega_r=abs(omega_r)
			omega_l=abs(omega_l)
	
		return RTC.RTC_OK

	
	#	##
	#	#
	#	# The aborting action when main logic error occurred.
	#	# former rtc_aborting_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onAborting(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The error action in ERROR state
	#	# former rtc_error_do()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onError(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The reset action that is invoked resetting
	#	# This is same but different the former rtc_init_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onReset(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The state update action that is invoked after onExecute() action
	#	# no corresponding operation exists in OpenRTm-aist-0.2.0
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#

	#	#
	#def onStateUpdate(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The action that is invoked when execution context's rate is changed
	#	# no corresponding operation exists in OpenRTm-aist-0.2.0
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onRateChanged(self, ec_id):
	#
	#	return RTC.RTC_OK
	



def Wheel_ControlInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=wheel_control_spec)
    manager.registerFactory(profile,
                            Wheel_Control,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    Wheel_ControlInit(manager)

    # Create a component
    comp = manager.createComponent("Wheel_Control")

def main():
	mgr = OpenRTM_aist.Manager.init(sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

